﻿local E, L, V, P, G = unpack(ElvUI)
local EP = LibStub("LibElvUIPlugin-1.0")
local Tuk = E:NewModule("Tuk_Style")
local LSM = LibStub('LibSharedMedia-3.0')
local CH = E:GetModule('Chat')
local LO = E:GetModule('Layout')
local S = E:GetModule('Skins')
local addon = ...

V["tukui"] = {
	["actionbars"] = true,
	["chat"] = true,
	["forcebordercolor"] = true,
	['unitframes'] = {
		["player"] = true,
		["pet"] = true,
		["target"] = true,
		["targettarget"] = true,
		["party"] = true,
		["raid"] = true,
		["raid40"] = true,
		["singleHeight"] = 25,
		["groupHeight"] = 20,
		["threat"] = false,
	},
}

P["tukui"] = {
	["installed"] = nil
}

Tuk.Version = GetAddOnMetadata('ElvUI_TukUIStyle', 'Version')

-- Objective Tracker Button
local function MinimizeButton_OnClick(self)
	local text = self.Text
	local symbol = text:GetText()
	
	if (symbol and symbol == '-') then
		text:SetText('+')
	else
		text:SetText('-')
	end
end

local function SkinObjeciveTracker()
	local button = ObjectiveTrackerFrame.HeaderMenu.MinimizeButton
	S:HandleButton(button)
	button:Size(16, 12)
	button.Text = button:CreateFontString(nil, 'OVERLAY')
	button.Text:FontTemplate(nil, 11)
	button.Text:Point('CENTER', button, 'CENTER')
	button.Text:SetJustifyH('CENTER')
	button.Text:SetText('-')
	button:HookScript('OnClick', MinimizeButton_OnClick)
	
	-- Remove textures from Objective tracker
	local otFrames = {ObjectiveTrackerBlocksFrame.QuestHeader, ObjectiveTrackerBlocksFrame.AchievementHeader, ObjectiveTrackerBlocksFrame.ScenarioHeader, BONUS_OBJECTIVE_TRACKER_MODULE.Header}
	for _, frame in pairs(otFrames) do
		if frame then
			frame:StripTextures()
		end
	end
end

function Tuk:SetMoverPosition(frame, anchor, parent, point, x, y)
	if not _G[frame] then return end
	local frame = _G[frame]

	frame:ClearAllPoints()
	frame:SetPoint(anchor, parent, point, x, y)
end

function Tuk:RegisterMedia()
	E['media'].ufFont = LSM:Fetch('font', 'TukUI Unitframes')
	E['media'].abFont = LSM:Fetch('font', 'TukUI Actionbars')
end

function Tuk:StyleSettings()
	if E.private.general.pixelPerfect == true then E.private.general.pixelPerfect = false end
	E.PixelMode = false
	if E.private.tukui.forcebordercolor then
		E.db.general.bordercolor = { r = 0.6,g = 0.6,b = 0.6 }
		E:UpdateMedia()
		E:UpdateBorderColors()
	end

	if E.private.tukui.chat then
		E.db.chat.panelBackdrop = "HIDEBOTH"
		E.db.datatexts.leftChatPanel = true
		E.db.datatexts.rightChatPanel = true
		E.db.datatexts.panelTransparency = false
		if IsAddOnLoaded("ElvUI_SLE") then
			E.db.sle.datatext.chathandle = false
		end
	end

	CH:PositionChat(true)
	CH:UpdateAnchors()
	LO:ToggleChatPanels()
	E:GetModule('Bags'):PositionBagFrames()
	LO:SetDataPanelStyle()

	E.db.general.bottomPanel = false

	LO:BottomPanelVisibility()
end

function Tuk:Repositioning()
	if E.private.tukui.chat then
		Tuk:SetMoverPosition("LeftChatPanel", "BOTTOMLEFT", ElvUIParent, "BOTTOMLEFT", 40, 19)
		Tuk:SetMoverPosition("RightChatPanel", "BOTTOMRIGHT", ElvUIParent, "BOTTOMRIGHT", -40, 19)
	end
	
	if E.private.tukui.actionbars and E.private.tukui.chat then
		Tuk:SetMoverPosition("ElvUI_Bar1", "BOTTOM", ElvUIParent, "BOTTOM", 0, 16)
	end
end

function Tuk:MoverOff(frame)
	_G[frame] = nil
	E.CreatedMovers[frame] = nil
end

function Tuk:DisableMovers()
	if E.private.tukui.chat then
		Tuk:MoverOff("ExperienceBarMover")
		Tuk:MoverOff("ReputationBarMover")
		Tuk:MoverOff("LeftChatMover")
		Tuk:MoverOff("RightChatMover")
	end
	if E.private.tukui.actionbars and E.private.tukui.chat then
		Tuk:MoverOff("ElvAB_1")
	end
	
end

function Tuk:Initialize()

	Tuk:StyleSettings()
	Tuk:DisableMovers()
	Tuk:Repositioning()
	Tuk:RegisterMedia()
	
	if E.private.tukui.chat then
		Tuk:CreatePanels()
		Tuk:RepositionExpRep()
		Tuk:SizePanels()
		hooksecurefunc(CH, "PositionChat", Tuk.SizePanels)
	end
	
	if IsAddOnLoaded("ElvUI_SLE") then
		local UB = E:GetModule('SLE_UIButtons')
		UB.Holder:CreateDropdownButton("Addon", "TukStyle", "|cffff8000TukUI Style|r", nil, nil, function() E:ToggleConfig(); LibStub("AceConfigDialog-3.0-ElvUI"):SelectGroup("ElvUI", "TukUIStyle") end, "ElvUI_TukUIStyle")
		UB.Holder:FrameSize()
	end
	
	SkinObjeciveTracker()
	
	local installRun = true
	if IsAddOnLoaded("ElvUI_SLE") and not E.private.sle.install_complete then installRun = false end
	if IsAddOnLoaded("ElvUI_BenikUI") and not E.db.bui.installed then installRun = false end
	if E.private.install_complete and E.private.tukui_installed == nil and installRun then
		Tuk:Setup()
	end

	EP:RegisterPlugin(addon, Tuk.Options)
end

E:RegisterModule(Tuk:GetName())