local E, L, V, P, G, _ = unpack(ElvUI); --Inport: Engine, Locales, PrivateDB, ProfileDB, GlobalDB, Localize Underscore
local Tuk = E:GetModule("Tuk_Style")
local locale = GetLocale()

local CURRENT_PAGE = 0
local MAX_PAGE = 3

local function SetupLayout()
	-- General
	E.db.general.minimap.size = 160

	-- Datatexts
	E.db.datatexts.font = 'ElvUI Font'
	E.db.datatexts.fontOutline = 'OUTLINE'
	E.db.datatexts.fontSize = 12

	-- Unitframes
	table.wipe(E.db.unitframe.units.player)
	E:CopyTable(E.db.unitframe.units.player, P.unitframe.units.player)
	table.wipe(E.db.unitframe.units.target)
	E:CopyTable(E.db.unitframe.units.target, P.unitframe.units.target)
	table.wipe(E.db.unitframe.units.targettarget)
	E:CopyTable(E.db.unitframe.units.targettarget, P.unitframe.units.targettarget)

	E.db.unitframe.fontSize = 17
	if locale == "ruRU" then
		E.db.unitframe.font = "ElvUI Font"
	else
		E.db.unitframe.font = "TukUI Unitframes"
	end
	E.db.unitframe.fontOutline = "OUTLINE"
	E.db.unitframe.colors.healthclass = true
	E.db.unitframe.colors.transparentHealth = true

	E.db.unitframe.units.player.health.yOffset = -46
	E.db.unitframe.units.player.power.yOffset = -46
	E.db.unitframe.units.player.debuffs.fontSize = 14
	E.db.unitframe.units.player.buffs.fontSize = 14	

	E.db.unitframe.units.target.name.position = "LEFT"
	E.db.unitframe.units.target.name.text_format = "[powercolor][power:current]  [namecolor][name:medium] [difficultycolor][smartlevel] [shortclassification]"
	E.db.unitframe.units.target.name.yOffset = -46
	E.db.unitframe.units.target.health.yOffset = -46
	E.db.unitframe.units.target.power.text_format = ""
	E.db.unitframe.units.target.debuffs.fontSize = 14
	E.db.unitframe.units.target.buffs.fontSize = 14

	E.db.unitframe.units.targettarget.power.height = 5
	E.db.unitframe.units.targettarget.height = 40
	E.db.unitframe.units.targettarget.name.yOffset = -37

	-- Actionbars
	E.db.actionbar.font = "TukUI Actionbars"
	E.db.actionbar.fontSize = 11
	E.db.actionbar.fontOutline = "OUTLINE"

	-- Auras
	E.db.auras.font = "TukUI Actionbars"
	E.db.auras.fontOutline = "OUTLINE"
	E.db.auras.fontSize = 10
	E.db.auras.consolidatedBuffs.enable = false
	E.db.auras.buffs.horizontalSpacing = 4
	E.db.auras.buffs.size = 28
	E.db.auras.debuffs.size = 30

	-- movers
	Tuk:SetMoverPosition('ElvUF_PlayerMover', 'BOTTOM', ElvUIParent, 'BOTTOM', -280, 175)
	Tuk:SetMoverPosition('ElvUF_TargetMover', 'BOTTOM', ElvUIParent, 'BOTTOM', 280, 175)
	Tuk:SetMoverPosition('ElvUF_TargetTargetMover', 'BOTTOM', ElvUIParent, 'BOTTOM', 0, 157)
	Tuk:SetMoverPosition('ElvUF_PetMover', 'BOTTOM', ElvUIParent, 'BOTTOM', 0, 238)
	Tuk:SetMoverPosition('BossButton', 'BOTTOM', ElvUIParent, 'BOTTOM', 0, 343)
	Tuk:SetMoverPosition('MinimapMover', 'TOPRIGHT', ElvUIParent, 'TOPRIGHT', -21, -16)
	Tuk:SetMoverPosition('BuffsMover', 'TOPRIGHT', ElvUIParent, 'TOPRIGHT', -189, -16)
	Tuk:SetMoverPosition('DebuffsMover', 'TOPRIGHT', ElvUIParent, 'TOPRIGHT', -189, -157)

	if InstallStepComplete then
		InstallStepComplete.message = L['Layout Set']
		InstallStepComplete:Show()
	end
	E:UpdateAll(true)
end

local function ResetAll()
	InstallNextButton:Disable()
	InstallPrevButton:Disable()
	InstallOption1Button:Hide()
	InstallOption1Button:SetScript('OnClick', nil)
	InstallOption1Button:SetText('')
	InstallOption2Button:Hide()
	InstallOption2Button:SetScript('OnClick', nil)
	InstallOption2Button:SetText('')
	InstallOption3Button:Hide()
	InstallOption3Button:SetScript('OnClick', nil)
	InstallOption3Button:SetText('')	
	InstallOption4Button:Hide()
	InstallOption4Button:SetScript('OnClick', nil)
	InstallOption4Button:SetText('')
	TukInstallFrame.SubTitle:SetText('')
	TukInstallFrame.Desc1:SetText('')
	TukInstallFrame.Desc2:SetText('')
	TukInstallFrame.Desc3:SetText('')
	TukInstallFrame:Size(550, 400)
end

local function InstallComplete()
	E.private.install_complete = E.version
	E.private.tukui_installed = true

	if GetCVarBool('Sound_EnableMusic') then
		StopMusic()
	end

	ReloadUI()
end

local function SetPage(PageNum)
	CURRENT_PAGE = PageNum
	ResetAll()
	InstallStatus:SetValue(PageNum)

	local f = TukInstallFrame

	if PageNum == MAX_PAGE then
		InstallNextButton:Disable()
	else
		InstallNextButton:Enable()
	end
	
	if PageNum == 1 then
		InstallPrevButton:Disable()
	else
		InstallPrevButton:Enable()
	end

	if PageNum == 1 then
		f.SubTitle:SetText(format(L['Welcome to Tukui Style version %s, for ElvUI %s.'], Tuk.Version, E.version))
		f.Desc1:SetText(L["By pressing the Continue button, Tukui Style will be applied in your current ElvUI installation."])
		f.Desc2:SetText(L['Please press the continue button to go onto the next step.'])
		InstallOption1Button:Show()
		InstallOption1Button:SetScript('OnClick', InstallComplete)
		InstallOption1Button:SetText(L['Skip Process'])
	elseif PageNum == 2 then
		f.SubTitle:SetText(L['Tukui Style Layout'])
		f.Desc1:SetText(L['This part of the installation changes the default ElvUI look. Doing this will also reset settings for some of your unitframe to default.'])
		f.Desc2:SetText(L['Please click the button below to apply the new layout.'])
		f.Desc3:SetText(L['Importance: |cff07D400High|r'])
		InstallOption1Button:Show()
		InstallOption1Button:SetScript('OnClick', SetupLayout)
		InstallOption1Button:SetText(L['Setup Layout'])
	elseif PageNum == 3 then
		f.SubTitle:SetText(L['Installation Complete'])
		f.Desc1:SetText(L['You are now finished with the installation process. If you are in need of technical support please visit us at http://www.tukui.org.'])
		f.Desc2:SetText(L['Please click the button below so you can setup variables and ReloadUI.'])
		InstallOption1Button:Show()
		InstallOption1Button:SetScript('OnClick', InstallComplete)
		InstallOption1Button:SetText(L['Finished'])
		TukInstallFrame:Size(550, 350)
		if InstallStepComplete then
			InstallStepComplete.message = L['Tukui Style Installed']
			InstallStepComplete:Show()
		end
	end
end

local function NextPage()
	if CURRENT_PAGE ~= MAX_PAGE then
		CURRENT_PAGE = CURRENT_PAGE + 1
		SetPage(CURRENT_PAGE)
	end
end

local function PreviousPage()
	if CURRENT_PAGE ~= 1 then
		CURRENT_PAGE = CURRENT_PAGE - 1
		SetPage(CURRENT_PAGE)
	end
end

function Tuk:Setup()
	if not InstallStepComplete then
		local imsg = CreateFrame('Frame', 'InstallStepComplete', E.UIParent)
		imsg:Size(418, 72)
		imsg:Point('TOP', 0, -190)
		imsg:Hide()
		imsg:SetScript('OnShow', function(self)
			if self.message then 
				PlaySoundFile([[Sound\Interface\LevelUp.ogg]])
				self.text:SetText(self.message)
				UIFrameFadeOut(self, 3.5, 1, 0)
				E:Delay(4, function() self:Hide() end)	
				self.message = nil
			else
				self:Hide()
			end
		end)

		imsg.firstShow = false

		imsg.bg = imsg:CreateTexture(nil, 'BACKGROUND')
		imsg.bg:SetTexture([[Interface\LevelUp\LevelUpTex]])
		imsg.bg:SetPoint('BOTTOM')
		imsg.bg:Size(326, 103)
		imsg.bg:SetTexCoord(0.00195313, 0.63867188, 0.03710938, 0.23828125)
		imsg.bg:SetVertexColor(1, 1, 1, 0.6)

		imsg.lineTop = imsg:CreateTexture(nil, 'BACKGROUND')
		imsg.lineTop:SetDrawLayer('BACKGROUND', 2)
		imsg.lineTop:SetTexture([[Interface\LevelUp\LevelUpTex]])
		imsg.lineTop:SetPoint('TOP')
		imsg.lineTop:Size(418, 7)
		imsg.lineTop:SetTexCoord(0.00195313, 0.81835938, 0.01953125, 0.03320313)

		imsg.lineBottom = imsg:CreateTexture(nil, 'BACKGROUND')
		imsg.lineBottom:SetDrawLayer('BACKGROUND', 2)
		imsg.lineBottom:SetTexture([[Interface\LevelUp\LevelUpTex]])
		imsg.lineBottom:SetPoint('BOTTOM')
		imsg.lineBottom:Size(418, 7)
		imsg.lineBottom:SetTexCoord(0.00195313, 0.81835938, 0.01953125, 0.03320313)

		imsg.text = imsg:CreateFontString(nil, 'ARTWORK')
		imsg.text:FontTemplate(nil, 32)
		imsg.text:Point('CENTER', 0, -4)
		imsg.text:SetTextColor(1, 0.82, 0)
		imsg.text:SetJustifyH('CENTER')
	end

	--Create Frame
	if not TukInstallFrame then
		local f = CreateFrame('Button', 'TukInstallFrame', E.UIParent)
		f.SetPage = SetPage
		f:Size(550, 400)
		f:SetTemplate('Transparent')
		f:SetPoint('CENTER')
		f:SetFrameStrata('TOOLTIP')

		f.Title = f:CreateFontString(nil, 'OVERLAY')
		f.Title:FontTemplate(nil, 17, nil)
		f.Title:Point('TOP', 0, -5)
		f.Title:SetText(L['Installation'])

		f.Next = CreateFrame('Button', 'InstallNextButton', f, 'UIPanelButtonTemplate')
		f.Next:StripTextures()
		f.Next:SetTemplate('Default', true)
		f.Next:Size(110, 25)
		f.Next:Point('BOTTOMRIGHT', -5, 5)
		f.Next:SetText(CONTINUE)
		f.Next:Disable()
		f.Next:SetScript('OnClick', NextPage)
		E.Skins:HandleButton(f.Next, true)

		f.Prev = CreateFrame('Button', 'InstallPrevButton', f, 'UIPanelButtonTemplate')
		f.Prev:StripTextures()
		f.Prev:SetTemplate('Default', true)
		f.Prev:Size(110, 25)
		f.Prev:Point('BOTTOMLEFT', 5, 5)
		f.Prev:SetText(PREVIOUS)
		f.Prev:Disable()
		f.Prev:SetScript('OnClick', PreviousPage)
		E.Skins:HandleButton(f.Prev, true)

		f.Status = CreateFrame('StatusBar', 'InstallStatus', f)
		f.Status:SetFrameLevel(f.Status:GetFrameLevel() + 2)
		f.Status:CreateBackdrop('Default')
		f.Status:SetStatusBarTexture(E['media'].normTex)
		f.Status:SetStatusBarColor(unpack(E['media'].rgbvaluecolor))
		f.Status:SetMinMaxValues(0, MAX_PAGE)
		f.Status:Point('TOPLEFT', f.Prev, 'TOPRIGHT', 6, -2)
		f.Status:Point('BOTTOMRIGHT', f.Next, 'BOTTOMLEFT', -6, 2)
		f.Status.text = f.Status:CreateFontString(nil, 'OVERLAY')
		f.Status.text:FontTemplate()
		f.Status.text:SetPoint('CENTER')
		f.Status.text:SetText(CURRENT_PAGE..' / '..MAX_PAGE)
		f.Status:SetScript('OnValueChanged', function(self)
			self.text:SetText(self:GetValue()..' / '..MAX_PAGE)
		end)

		f.Option1 = CreateFrame('Button', 'InstallOption1Button', f, 'UIPanelButtonTemplate')
		f.Option1:StripTextures()
		f.Option1:Size(160, 30)
		f.Option1:Point('BOTTOM', 0, 45)
		f.Option1:SetText('')
		f.Option1:Hide()
		E.Skins:HandleButton(f.Option1, true)

		f.Option2 = CreateFrame('Button', 'InstallOption2Button', f, 'UIPanelButtonTemplate')
		f.Option2:StripTextures()
		f.Option2:Size(110, 30)
		f.Option2:Point('BOTTOMLEFT', f, 'BOTTOM', 4, 45)
		f.Option2:SetText('')
		f.Option2:Hide()
		f.Option2:SetScript('OnShow', function() f.Option1:SetWidth(110); f.Option1:ClearAllPoints(); f.Option1:Point('BOTTOMRIGHT', f, 'BOTTOM', -4, 45) end)
		f.Option2:SetScript('OnHide', function() f.Option1:SetWidth(160); f.Option1:ClearAllPoints(); f.Option1:Point('BOTTOM', 0, 45) end)
		E.Skins:HandleButton(f.Option2, true)

		f.Option3 = CreateFrame('Button', 'InstallOption3Button', f, 'UIPanelButtonTemplate')
		f.Option3:StripTextures()
		f.Option3:Size(100, 30)
		f.Option3:Point('LEFT', f.Option2, 'RIGHT', 4, 0)
		f.Option3:SetText('')
		f.Option3:Hide()
		f.Option3:SetScript('OnShow', function() f.Option1:SetWidth(100); f.Option1:ClearAllPoints(); f.Option1:Point('RIGHT', f.Option2, 'LEFT', -4, 0); f.Option2:SetWidth(100); f.Option2:ClearAllPoints(); f.Option2:Point('BOTTOM', f, 'BOTTOM', 0, 45)  end)
		f.Option3:SetScript('OnHide', function() f.Option1:SetWidth(160); f.Option1:ClearAllPoints(); f.Option1:Point('BOTTOM', 0, 45); f.Option2:SetWidth(110); f.Option2:ClearAllPoints(); f.Option2:Point('BOTTOMLEFT', f, 'BOTTOM', 4, 45) end)
		E.Skins:HandleButton(f.Option3, true)

		f.Option4 = CreateFrame('Button', 'InstallOption4Button', f, 'UIPanelButtonTemplate')
		f.Option4:StripTextures()
		f.Option4:Size(100, 30)
		f.Option4:Point('LEFT', f.Option3, 'RIGHT', 4, 0)
		f.Option4:SetText('')
		f.Option4:Hide()
		f.Option4:SetScript('OnShow', function() 
			f.Option1:Width(100)
			f.Option2:Width(100)

			f.Option1:ClearAllPoints(); 
			f.Option1:Point('RIGHT', f.Option2, 'LEFT', -4, 0); 
			f.Option2:ClearAllPoints(); 
			f.Option2:Point('BOTTOMRIGHT', f, 'BOTTOM', -4, 45)  
		end)
		f.Option4:SetScript('OnHide', function() f.Option1:SetWidth(160); f.Option1:ClearAllPoints(); f.Option1:Point('BOTTOM', 0, 45); f.Option2:SetWidth(110); f.Option2:ClearAllPoints(); f.Option2:Point('BOTTOMLEFT', f, 'BOTTOM', 4, 45) end)
		E.Skins:HandleButton(f.Option4, true)

		f.SubTitle = f:CreateFontString(nil, 'OVERLAY')
		f.SubTitle:FontTemplate(nil, 15, nil)
		f.SubTitle:Point('TOP', 0, -40)

		f.Desc1 = f:CreateFontString(nil, 'OVERLAY')
		f.Desc1:FontTemplate()
		f.Desc1:Point('TOPLEFT', 20, -75)
		f.Desc1:Width(f:GetWidth() - 40)
		
		f.Desc2 = f:CreateFontString(nil, 'OVERLAY')
		f.Desc2:FontTemplate()	
		f.Desc2:Point('TOPLEFT', 20, -125)
		f.Desc2:Width(f:GetWidth() - 40)

		f.Desc3 = f:CreateFontString(nil, 'OVERLAY')
		f.Desc3:FontTemplate()
		f.Desc3:Point('TOPLEFT', 20, -175)
		f.Desc3:Width(f:GetWidth() - 40)

		local close = CreateFrame('Button', 'InstallCloseButton', f, 'UIPanelCloseButton')
		close:SetPoint('TOPRIGHT', f, 'TOPRIGHT')
		close:SetScript('OnClick', function()
			f:Hide()
		end)
		E.Skins:HandleCloseButton(close)

		f.tutorialImage = f:CreateTexture('InstallTutorialImage', 'OVERLAY')
		f.tutorialImage:Size(256, 128)
		f.tutorialImage:SetTexture('Interface\\AddOns\\ElvUI_TukUIStyle\\media\\textures\\logo.blp')
		f.tutorialImage:Point('BOTTOM', 0, 70)

	end
	
	TukInstallFrame:Show()
	NextPage()
end